
/* definitions */

#define	FALSE		0
#define	TRUE		1
#define	TXSTALLED 	-2
#define	TXNOTRDY	-3
#define	TXSENT		-4
#define	NO_CHAR		-5

#include <stdio.h>	/* standard i/o stuff */
#include <io.h>		/* standard i/o stuff */
#include <time.h>	/* time stuff */
#include <bios.h>	/* bios stuff */


/***********************
/* TITLE:
/*	dl -- download ascii files to serial port
/*
/* DESCRIPTION:
/*	dl downloads files to the serial port while showing
/*	response from receiving device on std output (console)
/*
/* USAGE:
/*	df [-h] [-a] [-p PORT] [-s SPEED] [-c C] [-d NN]
/*		  [-t PP] [-u RR] file1 [fileN]
/*	
/* COMPLIER:
/*	MS Quick-C
/*
/* REVISIONS:
/*	980123-2.02 -- correct type problem with temp value (k)
/*			used for clock cycle delays
/*
/*	980131-2.03 -- correct type problem downloading tbas files
/*
/*
/***********************/

/* globals */

FILE	*f1, *fopen(), *fileop();		/* input file channel */


/* char	buff[5];	/* buffer for hex conversion */

unsigned int
	stall,		/* flag for stall */
	portbase,	/* base address for serial port */
	comport,	/* number ID of com port */
	speed,		/* baudrate */
	last_char, 	/* last character received from line */
	err_noendchar,	/* timeout without end of line char */
	done;		/* flag for done */




/***********************
/* TITLE:
/*	main
/*
/* DESCRIPTION:
/*	drives it all
/*
/***********************/

void main(argc,argv)
int	argc;		/* argument count */
char	*argv[];	/* argument pointers */
{

	int	i,		/* arg index */
		j,		/* for switch */
		m,
		autobaud,	/* autobaud flag */
		txval,		/* tx value sent */
		rxval,		/* rx response value */
		tmp,		/* tmp value */
		waittic,	/* wait tic value */
		eof_waittic,	/* eof_wait tic value */
		tic_ctr,	/* tic counter value */
		wcr_flag,	/* mark wait RET echo */
		c,		/* temp char */
		t;		/* temp for case */

	char	*p,		/* temp ptr */
		waitchar;	/* wait char */

	clock_t	k;		/* tmp clock time */

	stall = FALSE;		/* let it rip */
	autobaud = FALSE;	/* clear autobaud flag */
	wcr_flag = FALSE;	/* clear autobaud flag */
	portbase = 0x3f8;
	speed = 19200;		/* set default baudrate */
	waitchar = EOF;		/* set wait char to non value */
	waittic = 0;		/* zero wait tics */
	eof_waittic = 0; 	/* zero eof_wait tics */

	err_noendchar = FALSE;	/* no error yet	  */


	if (argc <= 1) {
		/* wrong # of args */
		puts("dl: 980131-2.03 usage: ");
  puts("dl [-p port] [-s speed] [-c waitchar] [-d waitchar_num]");
  puts("      [-t waittic] [-u eof-tics] file1 [fileN]\n");
  puts("dl: use \"dl -h\" to see help message.\n");
		exit();
	}

	/* get any flags */
	for (i = 1; *(p = argv[i]) == '-'; i++)
	  while (j =  *++p)
	    switch (j) {
		case 'a':
		case 'A':
			/* set autobaud flag */
			autobaud = TRUE ;
			break;

		case 'c':
		case 'C':
			/* get wait char, char value */
			waitchar = (char) *(argv[++i]);
			break;

		case 'd':
		case 'D':
			/* get wait char, numeric value */
			waitchar = (char) atoi(argv[++i]);
			break;

		case 'h':
		case 'H':
			/* print the help info */
			help_msg();
			exit();
			break;

		case 'p':
		case 'P':
			/* get port*/
			comport = atoi(argv[++i]);  /* ser port int */
			if (comport == 1)
				portbase = 0x3f8;
			else if (comport == 2)
				portbase = 0x2f8;
			else if (comport == 3)
				portbase = 0x2e8;
			else if (comport == 4)
				portbase = 0x2e0;
			else 
				portbase = 0x3f8;
			break;

		case 'r':
		case 'R':
			/* set wcr_flag, to wait for CR echo */
			wcr_flag = TRUE;
			break;

		case 's':
		case 'S':
			/* get */
			speed = atoi(argv[++i]);  /* str to int */
			break;

		case 't':
		case 'T':
			/* get wait tics*/
			waittic = atoi(argv[++i]);  /* wait tics */
			break;

		case 'u':
		case 'U':
			/* get end-of-file wait tics*/
			eof_waittic = atoi(argv[++i]);  /* wait tics */
			break;

		default:
			puts("dl: invalid flag ");
			puts(p);
			puts("\n");
			exit();
			break;
	    }

	intsp(speed);	/* init the serial port */

	if (autobaud == TRUE) { /* do autobaud routine	*/
	  /* send 3 spaces at 3sp/sec */
	  for (j = 1; j < 3 ; j++) {
		sputchar(' ');	/* output space */
		for (k = 1; k < 6 ; k++) {
		  m = clock();	/* get an init value */
		  while (m == clock()) {
		    /* do this while clock same */
		    rxchar();   /* do this for echo */
		    ctlc_exit();	/* exit if ^C found */
		  }
		}
	  }

	  /* wait 1 second for signon message to complete */
	  for (k = 1; k < 18 ; k++) {
		  m = clock();	/* get an init value */
		  while (m == clock()) {
		    /* do this while clock same */
		    rxchar();   /* do this for echo */
		    ctlc_exit();	/* exit if ^C found */
		  }
	  }

	  sputchar('\r');	/* output return */

	  /* wait 1 second for any other fallout */
	  for (k = 1; k < 18 ; k++) {
		  m = clock();	/* get an init value */
		  while (m == clock()) {
		    /* do this while clock same */
		    rxchar();   /* do this for echo */
		    ctlc_exit();	/* exit if ^C found */
		  }
	  }

	}


	/* get the data input file pointer */
	if (i >= argc) {
		puts("dl: no input file(s)\n");
		exit();
	}

	/* do all the files */
	while (i < argc) {
	  done = FALSE;		/* done is false to start */
	  f1 = fileop(argv[i++],"r");	/* get input channel */
	    do {  /* +++ 					 */
		  /* here's the big loop where I demonstrate how */
		  /* to abuse C and make it look like chicken	 */
		  /* scratch ASM programming			 */

	      K000:
		ctlc_exit();	/* exit if ^C found */
		txval = txchar();	/* send one char */
		if (txval == EOF) goto EndFile;
		if (txval == '\r') goto K010;
		do { } while ((rxval = rxchar()) != NO_CHAR) ;
			/* get chars until no more */
		goto K000;	/* send another char */


	/* if waitchar is null then wait selected amount	*/
	/* of time and move on without error.  If waitchar	*/
	/* is NOT null and it is not found within time limit	*/
	/* then flag error and go on.				*/

	      K010:	/* \r sent, do end of line processing */
		if (wcr_flag == TRUE) {
		  /* if wcr_flag true then wait till \r is rx-ed */
		  do { } while ((rxval = rxchar()) != '\r') ;
			/* wait until '\r' is echoed */
		}
	      K010a:
		tic_ctr = waittic;	/* re-init tic timer */
	      K011:
		if (tic_ctr == 0) { 
		  if (waitchar != EOF) {
printf("\ndl: timeout! never got expected end of line character\n");
		  	err_noendchar = TRUE;
		  }
		  goto K000;
 		}
		ctlc_exit();	/* exit if ^C found */
		/* wait one tic, get chars while waiting */
		k = clock();	/* get an inital value of clock */
		while (k == clock()) { /* do this while clock same */
			if ((rxval = rxchar()) != NO_CHAR) {
				goto K012; /* bailout if char found */
			}
		}
		tic_ctr--;	/* dec tic counter	*/
		goto K011;	/* continue		*/

	      K012:
		if (last_char == waitchar) {
			last_char = -2	;	/* clear last char */
			goto K000;		/* do another line */
		}
		goto K010a;	/* else just reinit timer and cont */


	/* at end of file continue to collect character from	*/
	/* downloaded device until there is no more output	*/
	/* for eof_waittic Tics					*/
			
	      EndFile:
		tic_ctr = eof_waittic;	/* eof wait time */
	      K021:
		ctlc_exit();	/* exit if ^C found */
		k = clock();	/* get an inital value of clock */
		while (k == clock()) { /* do this while clock same */
			if ((rxval = rxchar()) != NO_CHAR)
				goto EndFile; /* bailout if char found */
		}
		tic_ctr--;
		if (tic_ctr != 0) goto K021;
		if (err_noendchar == FALSE)
			printf("\n");
		else {
	printf("\n\ndl: *** ERRORS ENCOUNTERED ***\n");
	printf("dl: At least one timeout without expected");
	printf(" end of line char\n");
		}
		/* close the file */
		fclose(f1);
	  } while (txval != EOF);
	}
		
}

/*************************
/* TITLE:
/*	rxchar
/*
/* DESCRIPTION:
/*	get an rx char if available and process
/*
/* RETURNED:
/*	TRUE if char received, FALSE otherwise
/*
/* AUTHOR:
/*	pbn
/*
/**************************/

int rxchar()
{
	int	i, j;	/* temps */

	if (srbrdy() == TRUE) {
		i = sgetchar();	/* get the input char */
		i = i & 0x7f;	/* look only at low 7 bits */
		if (i == ('s' & 0x1f))
			stall = TRUE;	/* stall if ^S */
		else if (i == ('q' & 0x1f))
			stall = FALSE;	/* stall if ^Q */
		else {
			last_char = i;	/* save as last_char */
			putchar (i); /* show on console */
/* diagnostic printf("          RX %c %x\n", i, i); */
		}
		return(i);	/* char was received */
	}
	else {
		return(NO_CHAR);  /* char was NOT received */
	}
}

/*************************
/* TITLE:
/*	txchar
/*
/* DESCRIPTION:
/*	send a char on TX port if OK to do so
/*
/* RETURNED:
/*	EOF if end of file, or various
/*
/* AUTHOR:
/*	pbn
/*
/**************************/

int txchar()
{
	int	i;	/* temp */

	if (stall == FALSE) {
		if (stbrdy() == TRUE) {
			if ((done == TRUE) || ((i = getc(f1)) == EOF)) {
				done = TRUE;	/* done with this */
				return (EOF);	/* end of file */
			}
			i = i & 0x7f;	/* ensure 7 bits */
			if (i == '\n')
				i = '\r'; /*conv NL to RET */
			sputchar(i);	/* output the char */
/* diagnostic	printf("TX %c %x\n", i, i);		*/
			return(i);	/* report TX transmitted */
		}
		else
			return(TXNOTRDY);	/* report TX not rdy */
	}
	else
		return(TXSTALLED);	/* report stalled TX */
}

/*************************
/* TITLE:
/*	fileop
/*
/* DESCRIPTION:
/*	try to open file, print error and exit if can't
/*
/* ARGUMENTS:
/*	ptr to name and how to open
/*
/* RETURNED:
/*	ptr to file
/*
/* AUTHOR:
/*	pbn
/*
/**************************/

FILE *fileop(p,t)	/* get input channel */
char	*p,	/* name of file to open */
	*t;	/* type of file to open */
{
	FILE	*f;	/* defines for returns */

	f = fopen (p,t);	/* try to open the file */

	if (f == NULL) {
		printf("dl: can't open %s\n", p);
		exit();
	}
	return(f);
}

/****************************************************/
/*
/* NAME:
/*	serial.c -- serial I/O for the PC and clones
/*
/* DESCRIPTION:
/*	provides routines to init the serial port to
/*	a desired speed, to send and receive data
/*	to set-reset bit I/O and to read bit I/O
/*
/* AUTHOR:
/*	pbn	870801
/*	revised	920326
/*	revised	980123
/*
/****************************************************/

int	ser_ad,		/* serial port A data */
	ser_aie,	/* serial port A int enable */
	ser_aii,	/* serial port A int ident */
	ser_alc,	/* serial port A line control */
	ser_amc,	/* serial port A modem control */
	ser_als,	/* serial port A line status */
	ser_ams,	/* serial port A modem status */
	ser_asc,	/* serial port A scratch */
	ser_adl,	/* serial port A divisor low */
	ser_adh;	/* serial port A divisor high */

/****************************************************/
/*
/* NAME:
/*	intsp -- initialize the serial port
/*
/* DESCRIPTION:
/*	initialize the ACE serial port of the PC
/*	to the baudrate specified
/*
/* ARGUMENTS:
/*	baudrate in decimal
/*
/* AUTHOR:
/*	pbn
/*
/****************************************************/

intsp(br)
unsigned int	br;	/* serial port */
{
	int 	i;	/* temp */

	ser_ad  = portbase + 0;	/* serial port A data */
	ser_aie = portbase + 1;	/* serial port A int enable */
	ser_aii = portbase + 2;	/* serial port A int ident */
	ser_alc = portbase + 3;	/* serial port A line control */
	ser_amc = portbase + 4;	/* serial port A modem control */
	ser_als = portbase + 5;	/* serial port A line status */
	ser_ams = portbase + 6;	/* serial port A modem status */
	ser_asc = portbase + 7;	/* serial port A scratch */
	ser_adl = portbase + 0;	/* serial port A divisor low */
	ser_adh = portbase + 1;	/* serial port A divisor high */

	switch (br) {
		case 110:
			i = 1047;
			break;

		case 300:
			i = 384;
			break;

		case 600:
			i = 192;
			break;

		case 1200:
			i = 96;
			break;

		case 2400:
			i = 48;
			break;

		case 4800:
			i = 24;
			break;

		case 9600:
			i = 12;
			break;

		case 19200:
			i = 6;
			break;

		case 38400:
			i = 3;
			break;

		default:
			puts("bad baudrate\n");
			exit();

	}
	outp(ser_aie,(char) 0x00);	/* no interrupts */
	outp(ser_alc,(char) 0x80);	/* access baudrate */
	outp(ser_adl,(char) i%256);	/* set low divisor */
	outp(ser_adh,(char) i/256);	/* set high divisor */
	/* now disconnect baudrate registers, 8 bits , 1 stop bit */
	/* no parity, no break */
	outp(ser_alc,(char) 0x03);	/* as stated above */
	outp(ser_amc,(char) 0x03);	/* RTS and DTR on */

}

/****************************************************/
/*
/* NAME:
/*	sgetchar -- serial port getchar
/*
/* DESCRIPTION:
/*	get a charcter from the serial port
/*
/* AUTHOR:
/*	pbn
/*
/****************************************************/

sgetchar()
{
	int	i;	/* temporary */

	while (srbrdy() == FALSE)
		;	/* wait until data ready */

	i = inp(ser_ad);	/* get a char */
	i &= 0x7f;	/* only look at LSB */

	return (i);	/* give it back */
}

/****************************************************/
/*
/* NAME:
/*	sputchar -- serial port putchar
/*
/* DESCRIPTION:
/*	put a charcter to the serial port
/*
/* AUTHOR:
/*	pbn
/*
/****************************************************/

sputchar(c)
int	c;	/* char to put */
{

	while (stbrdy() == FALSE)
		;	/* wait until buffer ready */

	outp(ser_ad, (char) c);	/* put a char */
}

/****************************************************/
/*
/* NAME:
/*	srbrdy -- serial port RX buffer ready test
/*
/* DESCRIPTION:
/*	see if a character is waiting
/*
/* AUTHOR:
/*	pbn
/*
/****************************************************/

srbrdy()
{
	int	i,	/* port status */
		v;	/* value for return */
	
	i = inp(ser_als) & 0x1;	/* get the input */
	v = i ? TRUE : FALSE;	/* set return value */
	return (v);	/* give it back */
}

/****************************************************/
/*
/* NAME:
/*	stbrdy -- serial port TX buffer ready test
/*
/* DESCRIPTION:
/*	see if a character can be put in TX buffer
/*
/* AUTHOR:
/*	pbn
/*
/****************************************************/

stbrdy()
{
	int	i,	/* port status */
		v;	/* value for return */

	i = inp(ser_als) & 0x20;	/* get the input */
	v = i ? TRUE : FALSE;	/* set return value */
	return (v);	/* give it back */
}

/****************************************************/
/*
/* NAME:
/*	scts -- serial port CTS OK?
/*
/* DESCRIPTION:
/*	see if clear to send is ACTIVE
/*
/* AUTHOR:
/*	pbn
/*
/****************************************************/

scts()
{
	int	i,	/* port status */
		v;	/* value for return */
	
	i = inp(ser_ams) & 0x10;	/* get the input, but only CTS */
	v = i ? TRUE : FALSE;	/* set return value */
	return (v);	/* give it back */
}

/***************************************************
/*
/* TITLE:
/*	ctlc_exit -- control C check and exit
/*
/* DESCRIPTION:
/*	see if ^C has been keyboarded, exit if so
/*
/* AUTHOR:
/*	pbn
/*
/***************************************************/

int ctlc_exit()
{
	int	i;	/* temp value */

	i = kbhit();	/* see if key */
	if (i != 0) 	/* if so, see if ^C */
		i = (getchar() == 3) ? TRUE : FALSE ;
	else
		i = FALSE;

	if (i == TRUE) {
		/* ^C found, print a message */
		printf("\ndl: *** ^C on keyboard, exit\n");
		exit();	
		}
	else
		return (FALSE);
}

/***************************************************
/*
/* TITLE:
/*	tic_st_ch()
/*
/* DESCRIPTION:
/*	wait for tick
/*
/* AUTHOR:
/*	pbn
/*
/***************************************************/

tic_st_ch()
{
	int	i;	/* temp value */

	i = clock();	/* get an inital value */
	while (clock() == i)
		rxchar();	/* get char if there */
				/* for state change 		    */
}

/***************************************************
/*
/* TITLE:
/*	help_msg()
/*
/* DESCRIPTION:
/*	print the help message
/*
/* AUTHOR:
/*	pbn
/*
/***************************************************/

help_msg()
{
	int i;

	for (i = 0; i < 24; i++)
		puts("");

puts("dl -- Help message.  dl is a 'downloader' program used to");
puts("send data from a PC, via the PC's serial terminal, to some");
puts("external device.  dl can be configured to wait after each");
puts("transmitted RET character until a specific character is");
puts("received from the remote device, or timeout has expired,");
puts("before sending the next line.");

puts("");
puts("--press Enter to continue--");
getchar();
	for (i = 0; i < 24; i++)
		puts("");

puts("The command format is as follows....");
puts("");
puts("df [-h] [-a] [-p PORT] [-s SPEED] [-c C] [-d DD]");
puts("      [-t TT] [-u UU] file1 [fileN]");
puts("");
puts("-h causes the help message (this msg) to be printed, and exit.");
puts("-a causes the file download to be prefaced with a spacing");
puts("     autobaud routine for TBAS-52 (default is no autobaud).");
puts("-p PORT causes the program to use COM port PORT (1, 2, 3, 4).");
puts("     default is 1 for COM1.");
puts("-s SPEED sets the serial speed to standard rates (300, ... 38400)");
puts("     default is 19200.");
puts("-c C sets the character to wait for following a transmittted");
puts("     RET, default is 0xff.");
puts("-d DD is same as -c except that DD is a decimal number instead");
puts("     of a character constant.  This is useful where the wait-for");
printf("      character is %c (62) ",(char) 62);
puts("which is difficult to escape in the");
puts("     command line.  default is 0xff.");
puts("-r   causes program to wait for RET echo before looking for C");
puts("-t TT sets the number of tics to wait for the special character");
puts("     to be TT tics, where a a tic is 1/18 seconds, default is 0.");
puts("-u UU sets the number of tics to wait at EOF to echo output from");
puts("     device until no more output");
puts("");
puts("--press Enter to continue--");
getchar();
	for (i = 0; i < 24; i++)
		puts("");

puts("In the case where -t is used but not -c or -d, then dl will");
puts("pause for TT tics after each transmitted RET character and");
puts("will not generate an error message.");
puts("");
}
